@@ -1127,6 +1127,11 @@ class GlirProgram(GlirObject):
         self._unset_variables = self._get_active_attributes_and_uniforms()
         self._handles = {}
         self._known_invalid = set()
+        # Clear attributes and samplers to avoid stale references when variables
+        # change type (e.g., switching from attribute to uniform)
+        self._attributes = {}
+        self._samplers = {}
+
         self._linked = True
 
     def _get_active_attributes_and_uniforms(self):
@@ -1283,8 +1288,13 @@ class GlirProgram(GlirObject):
                 gl.glBindBuffer(gl.GL_ARRAY_BUFFER, vbo_handle)
                 gl.glEnableVertexAttribArray(attr_handle)
                 func(attr_handle, *args)
-                if divisor is not None:
-                    gl.glVertexAttribDivisor(attr_handle, divisor)
+                if hasattr(gl, "glVertexAttribDivisor"):
+                    gl.glVertexAttribDivisor(attr_handle, divisor or 0)
+                elif divisor is not None:
+                    logger.warning(
+                        'Instanced rendering is not supported by the current'
+                        f'backend ("{gl.current_backend.__name__}")'
+                    )
             else:
                 gl.glBindBuffer(gl.GL_ARRAY_BUFFER, 0)
                 gl.glDisableVertexAttribArray(attr_handle)
